SystemOrganization addCategory: #'System-PasswordManager'!

Object subclass: #PasswordManager
	instanceVariableNames: 'directory'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'System-PasswordManager'!

!PasswordManager commentStamp: '<historical>' prior: 0!
Simple password manager to avoid storing passwords in the image.!

!PasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 15:23'!
default
	^Default ifNil:[Default := self new]! !

!PasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:53'!
editPasswords
	^self default editPasswords! !

!PasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:14'!
passwordAt: location user: user
	^self default passwordAt: location user: user! !

!PasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:14'!
passwordAt: location user: user put: password
	^self default passwordAt: location user: user put: password! !

!PasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:37'!
queryPasswordAt: location user: user
	^self default queryPasswordAt: location user: user! !

!PasswordManager methodsFor: 'accessing' stamp: 'bf 11/7/2005 14:58'!
directory
	"Answer the directory in which to find the passwords"
	^directory ifNil:[ExternalSettings assuredPreferenceDirectory]! !

!PasswordManager methodsFor: 'accessing' stamp: 'ar 11/4/2005 15:01'!
directory: aDirectory
	"Indicate the directory in which to find the passwords"
	directory := aDirectory! !

!PasswordManager methodsFor: 'editing' stamp: 'ar 11/4/2005 16:48'!
editPasswords
	"Edit the passwords"
	(self directory fileNamesMatching: '*.pwd') do:[:fName|
		self editPasswordsIn: (self directory fullNameFor: fName).
	].! !

!PasswordManager methodsFor: 'editing' stamp: 'ar 11/4/2005 16:53'!
editPasswordsIn: pwdFile
	"Edit the passwords"
	| file data |
	file := FileStream readOnlyFileNamed: pwdFile.
	data := file contents.
	file close.
	UIManager default edit: data label: pwdFile accept:[:text|
		file := FileStream forceNewFileNamed: pwdFile.
		file nextPutAll: text asString.
		file close.
	].

! !

!PasswordManager methodsFor: 'queries' stamp: 'ar 11/4/2005 17:14'!
passwordAt: location user: user
	"Answer the password stored under the given key, or nil if none can be found"
	| file stream tokens pwdFile |
	(location indexOf: Character space) = 0 
		ifFalse:[^self error: 'Location must not contain spaces'].
	(user indexOf: Character space) = 0 
		ifFalse:[^self error: 'User name must not contain spaces'].
	pwdFile := user,'.pwd'.
	file := [self directory readOnlyFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNil:[^nil].
	[stream := file contents readStream] ensure:[file close].
	[stream atEnd] whileFalse:[
		tokens := stream nextLine findTokens: ' '.
		(tokens size = 2 and:[tokens first match: location]) ifTrue:[
			^(Base64MimeConverter mimeDecode: tokens last as: String)
		].
	].
	^nil! !

!PasswordManager methodsFor: 'queries' stamp: 'kph 6/3/2008 11:36'!
passwordAt: location user: user put: password
	"Store the password under the given key"
	| file tokens pwd in out done sz pwdFile |
	user ifNil:[ ^ self ].
	(location indexOf: Character space) = 0 
		ifFalse:[^self error: 'Key must not contain spaces'].
	(user indexOf: Character space) = 0 
		ifFalse:[^self error: 'Key must not contain spaces'].
	password ifNotNil:[
		pwd := (Base64MimeConverter mimeEncode: password readStream) contents.
	].
	pwdFile := user,'.pwd'.
	file := [self directory readOnlyFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNotNil:[
		[in := file contents readStream] ensure:[file close].
	] ifNil:[in := String new readStream].
	out := WriteStream on: (String new: 1000).

	done := pwd == nil. "if clearing passwords, we're done already"
	[in atEnd] whileFalse:[
		tokens := in nextLine findTokens: ' '.
		tokens size = 2 ifTrue:[
			(tokens first match: location) 
				ifTrue:[pwd ifNotNil:[out nextPutAll: location; space; nextPutAll: pwd; cr. done := true]]
				ifFalse:[out nextPutAll: tokens first; space; nextPutAll: tokens last; cr]]].
	done ifFalse:[out nextPutAll: location; space; nextPutAll: pwd; cr].

	file := [self directory forceNewFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNil:[^nil].
	[file nextPutAll: out contents.
	sz := file size] ensure:[file close].
	sz = 0 ifTrue:[self directory deleteFileNamed: pwdFile ifAbsent:[]].
! !

!PasswordManager methodsFor: 'queries' stamp: 'kph 2/26/2009 04:54'!
queryPasswordAt: location user: user
	"Answer the password for the given user/location. 
	If the password is absent, query the user if interactive."
	| pwd |
	"search for existing password"
 
	pwd := self passwordAt: location user: user.
	pwd ifNotNil:[^pwd].

	pwd := FillInTheBlank requestPassword: 'Password for "', user, '" at ', location.
	pwd isEmptyOrNil ifTrue:[^nil].
	(self confirm: 'Remember password for "', user, '" at ', location,'?') ifTrue:[
		self passwordAt: location user: user put: pwd.
	].
	^pwd
	
 ! !
